;; ___  ___                 ______                _    
;; |  \/  |                 | ___ \              | |   
;; | .  . | ___   ___  _ __ | |_/ /_ __ ___  __ _| | __
;; | |\/| |/ _ \ / _ \| '_ \| ___ \ '__/ _ \/ _` | |/ /
;; | |  | | (_) | (_) | | | | |_/ / | |  __/ (_| |   < 
;; \_|  |_/\___/ \___/|_| |_\____/|_|  \___|\__,_|_|\_\
;;                                                                                                       
;; Copyright (C) 2020  
;; This file is part of the Amstrad game: MoonBreak
;;
;; Authors:
;; Ricardo Antonio, Ruiz Falcó (@R1ckyman) (ricky.rfr@gmail.com)
;; Alejandro Vicente, Rodríguez Segado (@VicenteSegado) (alejandrors13091993@gmail.com)
;; Enrique Antonio, Cores Rodríguez (@QuiqueCores) (encorod@gmail.com)
;;
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;; 
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;; 
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
;;  ARRAY_STRUCTURE.H.S
;;      This file defines the macros for the creation of the entity and pointer handling arrays.
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================

;;==============================================================================================================================
;;==============================================================================================================================
;;  DEFINE LINKED LIST STRUCTURE
;;==============================================================================================================================
;;==============================================================================================================================
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Defines the structure of the entity array.
.macro DefineComponentArrayStructure_Size _Tname, _N, _ComponentSize
    _Tname'_num:         .db 0
    _Tname'_list:        .dw 0x0000
    _Tname'_free_list:   .dw _Tname'_array
    _Tname'_array::
        .ds _N * _ComponentSize
.endm

;;==============================================================================================================================
;;==============================================================================================================================
;;  DEFINE ARRAYS OF COMPONENTS
;;==============================================================================================================================
;;==============================================================================================================================
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Defines the pointers of the componente array pointer access table.
.macro DefineComponentPointerAccessTable _Tname, _suf, _N, _N_Cmps
    _Tname'_components'_suf'_ptr_pend::    .dw . + 2*_N_Cmps+ + _suf*2*_N + 2*_suf
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Zero-pad an array of size n.
.macro DefineComponentArray _N
    .rept _N
        .dw 0x0000
    .endm
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Defines the structure for the component handler.
.macro DefineComponentPointerTable _Tname, _N_Cmps, _N
    _c = 0
    ;;  Array containing pointers to component pointer arrays.
    _Tname'_access_table::
    .rept _N_Cmps
        DefineComponentPointerAccessTable _Tname, \_c, _N, _N_Cmps
        _c = _c + 1
    .endm
    ;;  Zero-fill the component array with two additional words for the
    ;;  next free position and a null pointer fot he end of the array.
    _Tname'_components::
   .rept _N_Cmps
        DefineComponentArray _N
        .dw 0x0000
        .dw 0x0000
    .endm
.endm

;;==============================================================================================================================
;;==============================================================================================================================
;;  DEFINE ARRAY STANDARD
;;==============================================================================================================================
;;==============================================================================================================================
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Defines the structure for a basic memory manager.
.macro DefineBasicStructureArray_Size _Tname, _N, _ComponentSize
    _Tname'_array::
        .ds _N * _ComponentSize
.endm

;;==============================================================================================================================
;;==============================================================================================================================
;;  DEFINE ARRAY FOR BULLETS
;;==============================================================================================================================
;;==============================================================================================================================
.macro DefineArray_Bullets _Tname
    _Tname'_num:    .db 3           ;;  Maximum bullets weapon.
    _Tname'_array:: .ds 6           ;;  2 bytes weapon speed.
.endm
