;; ___  ___                 ______                _    
;; |  \/  |                 | ___ \              | |   
;; | .  . | ___   ___  _ __ | |_/ /_ __ ___  __ _| | __
;; | |\/| |/ _ \ / _ \| '_ \| ___ \ '__/ _ \/ _` | |/ /
;; | |  | | (_) | (_) | | | | |_/ / | |  __/ (_| |   < 
;; \_|  |_/\___/ \___/|_| |_\____/|_|  \___|\__,_|_|\_\
;;                                                                                                       
;; Copyright (C) 2020  
;; This file is part of the Amstrad game: MoonBreak
;;
;; Authors:
;; Ricardo Antonio, Ruiz Falcó (@R1ckyman) (ricky.rfr@gmail.com)
;; Alejandro Vicente, Rodríguez Segado (@VicenteSegado) (alejandrors13091993@gmail.com)
;; Enrique Antonio, Cores Rodríguez (@QuiqueCores) (encorod@gmail.com)
;;
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;; 
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;; 
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
;;  ENTITY.H.S
;;      This file defines the parameters that the entities will contain and a macro for their creation.
;;      It also defines other utility parameters such as constants and masks.
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
.include "cmp/macros.h.s"

;;==============================================================================================================================
;;==============================================================================================================================
;;  ENTITY CONSTRUCTOR
;;==============================================================================================================================
;;==============================================================================================================================
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Defines a new entity structure.
.macro DefineCmp_Entity _type, _class, _cmps, _x, _y, _vx, _vy, _w, _h, _spr, _animation_ptr, _collide_with, _ai_parent_st, _ai_processing, _ai_sight, _att_sta
    .narg __argn
    .if __argn - 16 ;; Validate parameter number.
        .error 1
    .else
        .dw nullptr                             ;;  Next element pointer linked list.
        .db _type                               ;;  Entity type.
        .db _class                              ;;  Entity class.
        .db _cmps                               ;;  Entity componentes.
        .db _x, _y                              ;;  Entity coordinates.
        .db _x, _y                              ;;  Initial coordinates of the entity.
        .db _vx, _vy                            ;;  Entity speed.
        .db _w, _h                              ;;  Entity dimensions (width and height).
        .db _w+_x-1, _h+_y- 1                   ;;  Completion of the entity width and height.
        .dw 0xCCCC                              ;;  
        .dw _spr                                ;;  Entity sprite.
        .db _collide_with                       ;;  Entities with which it collides.
        .db _ai_parent_st, _ai_parent_st, 0x00  ;;  Ai states.
        .db _ai_processing, _ai_processing      ;;  Processing and counter.
        .dw nullptr  , nullptr                  ;;  Patrol position and vision direction.
        .db _ai_sight                           ;;  Entity's view range.
        .db _att_sta, _att_sta                  ;;  Current life and total life.
        .dw nullptr                             ;;  Pointer to the equipped weapon.
        .dw _animation_ptr                      ;;  Animation pointer.
        .db 0                                   ;;  Animation status.
    .endif
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Player constructor
player_cmps = e_cmp_render | e_cmp_movable | e_cmp_input | e_cmp_collisionable | e_cmp_animated
.macro DefineCmp_Player _x, _y, _spr, _animation_ptr
    DefineCmp_entity e_type_player, e_class_default, player_cmps, _x, _y, 2, 0, 4, 16, _spr, _animation_ptr, e_type_default, e_ai_st_default, 0, 0, 6
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Melee enemy constructor
movable_enemy = e_cmp_render | e_cmp_movable | e_cmp_collisionable | e_cmp_ai
.macro DefineCmp_MeleeEnemy _x, _y, _spr, _ai_parent_st
    DefineCmp_entity e_type_enemy, e_class_melee, movable_enemy, _x, _y, 0, 0, 4, 16, _spr, nullptr, e_type_default, _ai_parent_st, 10, 30, 3
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Short range enemy constructor
.macro DefineCmp_ShortRangeEnemy _x, _y, _spr, _ai_parent_st
    DefineCmp_entity e_type_enemy, e_class_short_range, movable_enemy, _x, _y, 0, 0, 4, 16, _spr, nullptr, e_type_default, _ai_parent_st, 10, 30, 3
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Short range enemy constructor
.macro DefineCmp_LongRangeEnemy _x, _y, _spr
    DefineCmp_entity e_type_enemy, e_class_long_range, movable_enemy, _x, _y, 0, 0, 5, 14, _spr, nullptr, e_type_default, e_ai_st_turret, 30, 20, 3
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Shot constructor
.macro DefineCmp_Shot _spr
    DefineCmp_entity e_type_shot, e_class_default, movable_enemy, 0, 0, 0, 0, 3, 6, _spr, nullptr, e_type_default, e_ai_st_bullet, 35, 20, 1
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Generated enemy constructor
.macro DefineCmp_Generated _x, _y, _spr
    DefineCmp_entity e_type_enemy, e_class_generated, movable_enemy, _x, _y, 0, 0, 4, 16, _spr, nullptr, e_type_default, e_ai_st_attack, 10, 20, 1
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Generator enemy constructor
.macro DefineCmp_Generator _x, _y, _spr, _ai_parent_st
    DefineCmp_entity e_type_enemy, e_class_generator, movable_enemy, _x, _y, 0, 0, 5, 14, _spr, nullptr, e_type_default, e_ai_st_guard_selector, 120, 20, 1
.endm

;;==============================================================================================================================
;;==============================================================================================================================
;;  ENTITY STRUCTURE
;;==============================================================================================================================
;;==============================================================================================================================
BeginStruct e
e_offset = 0                                
                                ;;  CMP: Linked list
Sfield e, ptr_l, 1
Sfield e, ptr_h, 1
                                ;;  CMP: Entity type
Sfield e, type,  1
Sfield e, class, 1
Sfield e, cmps,  1
                                ;;  CMP: Position
Sfield e, x, 1
Sfield e, y, 1
Sfield e, initialx, 1
Sfield e, initialy, 1
                                ;;  CMP: Velocity
Sfield e, vx, 1
Sfield e, vy, 1
                                ;;  CMP: Render
Sfield e, w, 1
Sfield e, h, 1
Sfield e, endx, 1
Sfield e, endy, 1
Sfield e, last_x, 1
Sfield e, last_y, 1
Sfield e, spr_l, 1
Sfield e, spr_h, 1
                                ;;  CMP: Collisions
Sfield e, collide_with, 1
                                ;;  CMP: AI
Sfield e, ai_parent_st, 1
Sfield e, ai_st, 1
Sfield e, ai_pre_st, 1
Sfield e, ai_processing, 1
Sfield e, ai_counter, 1
Sfield e, ai_aim_x, 1
Sfield e, ai_aim_y, 1
Sfield e, ai_patrol_step_l, 1
Sfield e, ai_patrol_step_h, 1
Sfield e, ai_sight, 1
                                ;;  CMP: Attributes
Sfield e, att_sta, 1
Sfield e, att_max_sta, 1
                                ;;  CMP: Weapons
Sfield e, equipped_weapon_l, 1
Sfield e, equipped_weapon_h, 1
                                ;;  CMP: Animation pointer
Sfield e, animation_ptr_l, 1
Sfield e, animation_ptr_h, 1
                                ;;  CMP: Animation status
Sfield e, animation_status, 1
                                ;;  Entity size
EndStruct e

;;==============================================================================================================================
;;==============================================================================================================================
;;  ENTITY CLASS MASKS AND BITS
;;==============================================================================================================================
;;==============================================================================================================================
e_class_explosive   = 0x80
e_class_teleporter  = 0x40
e_class_generated   = 0x20
e_class_generator   = 0x10
e_class_short_range = 0x04
e_class_long_range  = 0x02
e_class_melee       = 0x01
e_class_default     = 0x00

;;==============================================================================================================================
;;==============================================================================================================================
;;  ENTITY TYPE MASKS AND BITS
;;==============================================================================================================================
;;==============================================================================================================================
e_type_dead             = 0x08
e_type_shot             = 0x04
e_type_enemy            = 0x02
e_type_player           = 0x01
e_type_default          = 0x00

;;==============================================================================================================================
;;==============================================================================================================================
;;  ENTITY COMPONENT MASKS AND BITS
;;==============================================================================================================================
;;==============================================================================================================================
e_cmp_render        = 0x01    ;;    Renderable entity
e_cmp_movable       = 0x02    ;;    Entity with physical.
e_cmp_input         = 0x04    ;;    Entity controllable by input
e_cmp_ai            = 0x08    ;;    Entity controllable by ai
e_cmp_collisionable = 0x10    ;;    Collisionable entity
e_cmp_animated      = 0x20    ;;    Animated entity

;;==============================================================================================================================
;;==============================================================================================================================
;;  ENTITY COMPONENT ENUM
;;==============================================================================================================================
;;==============================================================================================================================
DefEnum e_cmpID
Enum e_cmpID AI
Enum e_cmpID Collisionable
Enum e_cmpID Num_Components

;;==============================================================================================================================
;;==============================================================================================================================
;;  ENTITY AI BEHAVIORS MASKS AND BITS
;;==============================================================================================================================
;;==============================================================================================================================
e_ai_st_guard               = 0x42
e_ai_st_guard_selector      = 0x41

e_ai_st_teleportation       = 0x28
e_ai_st_move_to             = 0x24
e_ai_st_patrol              = 0x22
e_ai_st_patrol_selector     = 0x21

e_ai_st_processing          = 0x18  
e_ai_st_heal                = 0x14
e_ai_st_escape              = 0x12
e_ai_st_attack              = 0x11

e_ai_st_bullet              = 0x04  ;; Encompasses self destruction and edge destruction.
e_ai_st_turret              = 0x01
e_ai_st_default             = 0x00

;;==============================================================================================================================
;;==============================================================================================================================
;;  UTILITY DEFINITIONS
;;==============================================================================================================================
;;==============================================================================================================================
nullptr         = 0x0000
no_stamina      = 0x00
max_entities    = 15