;; ___  ___                 ______                _    
;; |  \/  |                 | ___ \              | |   
;; | .  . | ___   ___  _ __ | |_/ /_ __ ___  __ _| | __
;; | |\/| |/ _ \ / _ \| '_ \| ___ \ '__/ _ \/ _` | |/ /
;; | |  | | (_) | (_) | | | | |_/ / | |  __/ (_| |   < 
;; \_|  |_/\___/ \___/|_| |_\____/|_|  \___|\__,_|_|\_\
;;                                                                                                       
;; Copyright (C) 2020  
;; This file is part of the Amstrad game: MoonBreak
;;
;; Authors:
;; Ricardo Antonio, Ruiz Falcó (@R1ckyman) (ricky.rfr@gmail.com)
;; Alejandro Vicente, Rodríguez Segado (@VicenteSegado) (alejandrors13091993@gmail.com)
;; Enrique Antonio, Cores Rodríguez (@QuiqueCores) (encorod@gmail.com)
;;
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;; 
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;; 
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
;;  WEAPON.H.S
;;      This file defines the parameters that the entities will contain and a macro for their creation.
;;      It also defines other utility parameters such as constants and masks.
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
.include "cmp/macros.h.s"

;;==============================================================================================================================
;;==============================================================================================================================
;;  WEAPON CONSTRUCTOR
;;==============================================================================================================================
;;==============================================================================================================================
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Defines a new weapon structure.
.macro DefineCmp_Weapon _type, _damage, _distance, _burst, _cadence, _vx, _vy
    .narg __argn
    .if __argn - 7 ;; Validate parameter number.
        .error 1
    .else
        .db _type
        .db _damage
        .db _distance
        .db _burst
        .db _cadence
        .db 0
        .db _vx
        .db _vy
    .endif
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Rifle constructor
.macro DefineCmp_Rifle
    DefineCmp_Weapon w_type_rifle,   1, 50, 1, 35, 2, 0
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Enemy rifle constructor
.macro DefineCmp_EnemyRifle
    DefineCmp_Weapon w_type_rifle,   1, 50, 1, 2, 2, 0
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Rifle shotgun constructor
.macro DefineCmp_Shotgun
    DefineCmp_Weapon w_type_shotgun, 2, 10, 3, 30, 2, 0
.endm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Enemy shotgun constructor
.macro DefineCmp_EnemyShotgun
    DefineCmp_Weapon w_type_shotgun, 2, 10, 3, 10, 2, 0
.endm

;;==============================================================================================================================
;;==============================================================================================================================
;;  WEAPON STRUCTURE
;;==============================================================================================================================
;;==============================================================================================================================
BeginStruct w
w_offset = 0       
                                        ;;  Weapon type.
Sfield w, type, 1
                                        ;;  Weapon damage.
Sfield w, damage, 1
                                        ;;  Shooting distance.
Sfield w, distance, 1
                                        ;;  Number of simultaneous bullets.
Sfield w, burst, 1
                                        ;;  Rate of fire.
Sfield w, cadence, 1
Sfield w, recover_cadence, 1
                                        ;;  Firing direction on both axes.
Sfield w, vx, 1
Sfield w, vy, 1
EndStruct w

;;==============================================================================================================================
;;==============================================================================================================================
;;  WEAPON TYPE MASKS AND BITS
;;==============================================================================================================================
;;==============================================================================================================================
w_type_invalid  =   0x00
w_type_rifle    =   0x01
w_type_shotgun  =   0x02

;;==============================================================================================================================
;;==============================================================================================================================
;;  UTILITY DEFINITIONS
;;==============================================================================================================================
;;==============================================================================================================================
player_weapons  = 2
max_weapons     = 10 + player_weapons - 1