;; ___  ___                 ______                _    
;; |  \/  |                 | ___ \              | |   
;; | .  . | ___   ___  _ __ | |_/ /_ __ ___  __ _| | __
;; | |\/| |/ _ \ / _ \| '_ \| ___ \ '__/ _ \/ _` | |/ /
;; | |  | | (_) | (_) | | | | |_/ / | |  __/ (_| |   < 
;; \_|  |_/\___/ \___/|_| |_\____/|_|  \___|\__,_|_|\_\
;;                                                                                                       
;; Copyright (C) 2020  
;; This file is part of the Amstrad game: MoonBreak
;;
;; Authors:
;; Ricardo Antonio, Ruiz Falcó (@R1ckyman) (ricky.rfr@gmail.com)
;; Alejandro Vicente, Rodríguez Segado (@VicenteSegado) (alejandrors13091993@gmail.com)
;; Enrique Antonio, Cores Rodríguez (@QuiqueCores) (encorod@gmail.com)
;;
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;; 
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;; 
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>
;;
;;  Animation manager
;;
.include "man/animation_manager.h.s"
.include "cmp/entity.h.s"
.module Animation_manager

animation_speed:: .db 0

_player_standing::
    .dw _sp_player_down_0
    .dw 0000         ;; Null pointer
    .dw _player_standing     ;; Animation pointer
_player_down::
    .dw _sp_player_down_0
    .dw _sp_player_down_1
    .dw _sp_player_down_2
    .dw 0000         ;; Null pointer
    .dw _player_down ;; Animation pointer
_player_up::
    .dw _sp_player_up_0
    .dw _sp_player_up_1
    .dw _sp_player_up_2
    .dw 0000         ;; Null pointer
    .dw _player_up ;; Animation pointer
_player_left::
    .dw _sp_player_left_0
    .dw _sp_player_left_1
    .dw _sp_player_left_2
    .dw 0000         ;; Null pointer
    .dw _player_left ;; Animation pointer
_player_right::
    .dw _sp_player_right_0
    .dw _sp_player_right_1
    .dw _sp_player_right_2
    .dw 0000         ;; Null pointer
    .dw _player_right ;; Animation pointer

_animation_array:
    .dw _player_standing
    .dw _player_left
    .dw _player_right
    .dw _player_up
    .dw _player_down

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Updates the animation of the entities, (only the player)
;;  INPUT:
;;      - IX
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
animation_manager_update::
    ;; If the entity does not have an animation, returns
    ld l, e_animation_ptr_l(ix)
    ld h, e_animation_ptr_h(ix)
    ld a, l
    or h
    ret z

    call animation_manager_calculate_status
    ld a, e_animation_status(ix)
    cp b
    jr z, _continue_animation
    _new_animation:
    ld e_animation_status(ix), a

    call animation_manager_get_animation
    ld e_animation_ptr_l(ix), l
    ld e_animation_ptr_h(ix), h

    _continue_animation:
    ld a, (animation_speed)
    dec a
    jr z, _process_animation
    _decrease_timer:
    ld (animation_speed), a
    ret
    _process_animation:
    ld a, #2
    ld (animation_speed), a

    ld c, (hl)
    inc hl
    ld b, (hl)
    inc hl

    ld a, c
    or b
    jr z, _reset_animation

    ld e_animation_ptr_l(ix), l
    ld e_animation_ptr_h(ix), h

    ld e_spr_l(ix), c
    ld e_spr_h(ix), b
    ret
_reset_animation:
    ld a, (hl)
    inc hl
    ld h, (hl)
    ld l, a
    jr _continue_animation

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Returns the animation in the animation array based on the status
;;  INPUT:
;;      - B: Animation status of the entity
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
animation_manager_get_animation::
    ld hl, #_animation_array
    ld a, b
    add a
    ;; HL + A
    add l
    ld l, a
    ld a, h
    adc #0  ;; Adds to register A #0 + carry
    ld h, a

    ld a, (hl)
    inc hl
    ld h, (hl)
    ld l, a
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Calculates the status of the animation based on the vx and vy parameters
;;  INPUT:
;;      - IX
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
animation_manager_calculate_status::
    ld b, #0
    ld a, e_vx(ix)
    ;; z -> equal, S & P/V equal -> positive, S & P/V not equal -> negative
    cp #0
    jr z, _no_vx
    _vx:
    jp m, _vxs_set
    _vx_s_reset:
    jp po, _positive_vx
    jr _negative_vx
    _vxs_set:
    jp pe, _positive_vx
    _negative_vx:
    inc b
    jr _calulated
    _positive_vx:
    ld b, #2
    jr _calulated
    _no_vx:
    ld a, e_vy(ix)
    cp #0
    jr z, _calulated
    _vy:
    jp m, _vy_s_set
    _vy_s_reset:
    jp po, _positive_vy
    jr _negative_vy
    _vy_s_set:
    jp pe, _positive_vy
    _negative_vy:
    ld b, #3
    jr _calulated
    _positive_vy:
    ld b, #4
    _calulated:
    ret