;; ___  ___                 ______                _    
;; |  \/  |                 | ___ \              | |   
;; | .  . | ___   ___  _ __ | |_/ /_ __ ___  __ _| | __
;; | |\/| |/ _ \ / _ \| '_ \| ___ \ '__/ _ \/ _` | |/ /
;; | |  | | (_) | (_) | | | | |_/ / | |  __/ (_| |   < 
;; \_|  |_/\___/ \___/|_| |_\____/|_|  \___|\__,_|_|\_\
;;                                                                                                       
;; Copyright (C) 2020  
;; This file is part of the Amstrad game: MoonBreak
;;
;; Authors:
;; Ricardo Antonio, Ruiz Falcó (@R1ckyman) (ricky.rfr@gmail.com)
;; Alejandro Vicente, Rodríguez Segado (@VicenteSegado) (alejandrors13091993@gmail.com)
;; Enrique Antonio, Cores Rodríguez (@QuiqueCores) (encorod@gmail.com)
;;
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;; 
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;; 
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>
.module Game_Manager

.include "sys/input_system.h.s"
.include "sys/render_system.h.s"
.include "sys/physics_system.h.s"
.include "sys/ai_control_system.h.s"
.include "sys/collision_detection_system.h.s"
.include "man/entity_manager.h.s"
.include "man/map_manager.h.s"
.include "man/patrol_manager.h.s"
.include "man/components_manager.h.s"
.include "man/game_manager.h.s"
.include "man/weapon_manager.h.s"
.include "man/world_manager.h.s"
.include "man/menu_manager.h.s"
.include "man/hud_manager.h.s"
.include "man/animation_manager.h.s"
.include "man/o2_manager.h.s"
.include "cmp/entity.h.s"
.include "cmp/weapon.h.s"
.include "cmp/array_structure.h.s"
.include "cpctelera.h.s"

.globl _sp_bullet
.globl _sp_melee_enemy

;; To use when picking up ship pieces
;;    call world_manager_unlock_door_level_1
;;    call world_manager_unlock_door_level_2
;;    call world_manager_unlock_door_level_3

;;==============================================================================================================================
;;==============================================================================================================================
;;  CONSTANT
;;==============================================================================================================================
;;==============================================================================================================================
changing_weapon:: .db 0
cooldown_changing_weapon = 5
state:          .db main_menu   ;; Game state ----- 0: Game loop, 1: Main menu, 2: Map loading, 3: Pause menu, 4: Game over, 5: Victory

;;==============================================================================================================================
;;==============================================================================================================================
;;  ENTITY TEMPLATES
;;      _type, _class, _cmps, _x, _y, _vx, _vy, _w, _h, _spr, _collide_with, _ai_parent_st, _ai_processing, _ai_sight, _att_sta
;;==============================================================================================================================
;;==============================================================================================================================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Player, generated enemy and bullet templates.
player:           DefineCmp_Player          40, 40,  _sp_player_down_0, _player_down
shot:             DefineCmp_Shot                     _sp_bullet
generatedEnemy:   DefineCmp_Generated       60, 10,  _sp_melee_enemy

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Bullet array.
DefineArray_Bullets _bullet

;;==============================================================================================================================
;;==============================================================================================================================
;;  FUNCTIONS
;;==============================================================================================================================
;;==============================================================================================================================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  GAME_MANAGER::Update
;;		Initialize all systems
;;  INPUT:
;;  MODIFY:
;; 
game_manager_init::
    call render_system_init
    call world_manager_init
    call entity_manager_init
    call input_system_init
    call ai_control_system_init
    call entity_manager_getEntityArrayIX
    call physics_system_init
    call collision_detection_system_init
    call game_manager_create_starting_entities
    call patrol_manager_init
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Initializes and show the main menu screen
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_manager_main_menu::
    call menu_manager_init
    call world_manager_init_level
    cpctm_clearScreen_asm 0x00
    call hud_manager_init
    call o2_manager_init
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Initializes and show the pause menu screen
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_manager_pause_menu::
    call game_manager_reset
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Initializes and show the game over screen
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_manager_game_over::
    ;; Show game over screen
    call game_manager_reset
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Initializes and show the victory screen
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_manager_victory::
    ;; Show victory screen
    call game_manager_reset
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Draws the current map
;;  INPUT:
;;  MODIFY:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_manager_load_map::
    call map_manager_set_to_map
    call map_manager_draw
    call map_manager_set_to_restore
    ld a, #game_loop
    ld (state), a
    call world_manager_init_map
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  GAME_MANAGER::Create_Starting_Entities
;;		Resets the system
;;  INPUT:
;;  MODIFY:
;;
game_manager_create_starting_entities::
    ld hl, #player
    call entity_manager_create
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  GAME_MANAGER::Reset
;;		Resets the system
;;  INPUT:
;;  MODIFY:
;; 
game_manager_reset::
    ld a, #main_menu
    ld (state), a
    call input_system_init
    call entity_manager_init
    call ai_control_system_init
    call entity_manager_getEntityArrayIX
    call physics_system_init
    call collision_detection_system_init
    call game_manager_create_starting_entities
    call patrol_manager_init
    call world_manager_reset
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  GAME_MANAGER::Update
;;		Upgrade the input system, ai system, physics system and collision system.
;;  INPUT:
;;  MODIFY:
;;  RETURNS:
;; 
game_manager_update::
    call input_system_update
    call ai_control_system_update
    call entity_manager_getEntityArrayIX
    call physics_system_update
    call collision_detection_system_update
    call o2_manager_update
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  GAME_MANAGER::Render
;;		Update render system
;;  INPUT:
;;  MODIFY:
;; 
game_manager_render::
    call entity_manager_getEntityArrayIX
    call render_system_update
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  GAME_MANAGER::Update_Entities
;;		Update the vector of entities destroying the marked ones.
;;  INPUT:
;;  MODIFY:
;; 
game_manager_update_entities::
    call entity_manager_update
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Gets the current state of the game loop
;;  MODIFIES:
;;      - A: Returns state
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_manager_get_state::
    ld a, (state)
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Sets the current state of the game loop
;;  MODIFIES:
;;      - A: Returns state
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_manager_set_state::
    ld (state), a
    ret

;;==============================================================================================================================
;;==============================================================================================================================
;;  SHOOTING FUNCTIONS
;;==============================================================================================================================
;;==============================================================================================================================
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  GAME_MANAGER::shot_again
;;      Check if the gun can fire again.
;;  INPUT:
;;		IY: Pointer to the weapon.
;;       A: w_recover_cadence(iy)
;;  MODIFY: A, E
;;  RETURNS:
;;      z = 0 if the rate of the weapon has not been recovered. \ z = 1 if the rate of weapon has been recovered.
;;
game_manager_shot_again::
    ;;  Check if the value of W_RECOVER_CADENCE is equal to that of W_CADENCE.
    ;;  If it is the same, W_RECOVER_CADENCE resets to zero and it is possible to fire again.
    ;;  If it is different, the value of W_RECOVER_CADENCE is increased.
    inc a
    ld w_recover_cadence(iy), a
    ld e, w_cadence(iy)
    sub e
    ret nz
    ld w_recover_cadence(iy), #0
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  GAME_MANAGER::setBulletSpeeds
;;      Fill the bullet array with velocities for two additional bullets.
;;  INPUT:
;;		IY: Pointer to gun.
;;  MODIFY:
;;      A, DE
;;  RETURNS:
;;
game_manager_setBulletSpeeds::
    ld hl, #_bullet_array

    ;;  The current speed of weapon is saved (speed that the bullet will catch).
    ld e, w_vx(iy)
    ld d, w_vy(iy)

    ;;  Frist bullet speeds.
    ld (hl), e  ;; first bullet vx
    inc hl
    ld (hl), d  ;; first bullet vy
    inc hl

    ;;  If the weapon is not a shotgun, no more bullets are needed and it ends.
    ld a, w_type(iy)
    cp #w_type_shotgun
    ret nz

    ;;  If the weapon is a shotgun, the vector is filled with velocities for two additional bullets.
    ;;  The speed of these bullets will be such that the first bullet is fired to the center and these two
    ;;  to its diagonals.
    ld a, e
    or a
    jr nz, __vx_diff_zero
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;  FIRST BULLET    SECOND BULLET   THIRD BULLET
    ;;    VX  VY           VX  VY          VX  VY
    ;;     2   0            2  -4           2   4
    ;;     0   4            2   4          -2   4
    ;;     2   4            2   0           0   4
    ;;  The second and third bullets have been calculated based on the condition of vx in the first bullet
    ;;  followin the previus pattern.
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    __vx_zero:
    add #2
    ld (hl), a  ;; second bullet vx
    inc hl

    ld (hl), d  ;; second bullet vy
    inc hl

    ld a, e
    sub #2
    ld (hl), a  ;; third bullet vx
    inc hl

    ld (hl), d  ;; thied bullet vy
    ret

    __vx_diff_zero:
    ld (hl), e  ;; second bullet vx
    inc hl

    ld a, d
    sub #4
    ld (hl), a  ;; second bullet vy
    inc hl

    ld (hl), e  ;; third bullet vx
    inc hl

    ld a, d
    add #4
    ld (hl), a  ;; third bullet vy
    ret
    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  GAME_MANAGER::change_weapon
;;      The entity's weapon is changed depending on the one it had previously.
;;      If he had a shotgun switch to a rifle.
;;      If he had a rifle, switch to a shotgun.
;;  INPUT:
;;		IY: Pointer to player.
;;  MODIFY:
;;      A, DE, IX, BC, IY
;;  RETURNS:
;;
game_manager_change_weapon::
    ;;  Waiting time to change weapons.
    ld a, (changing_weapon)
    or a
    jr nz, __cooldown

    call weapon_manager_getWeaponArrayHL

    ;;  DE = Pointer to equipped weapon.
    ld e, e_equipped_weapon_l(iy)
    ld d, e_equipped_weapon_h(iy)

    ;;  IX = DE
    ld__ixl_e
    ld__ixh_d

    ;;  The type of weapon is checked.
    ld a, w_type(ix)
    cp #w_type_shotgun
    jr z, __switch_to_rifle

;;  Equipped weapon was rifle and changed to shotgun.
__switch_to_shotgun:
    ld bc, #sizeof_w
    add hl, bc

;;  Equipped weapon was shotgun and changed to rifle.
__switch_to_rifle:
    ld c, w_vx(ix)
    ld b, w_vy(ix)

    ld e_equipped_weapon_l(iy), l
    ld e_equipped_weapon_h(iy), h

    ld e, e_equipped_weapon_l(iy)
    ld d, e_equipped_weapon_h(iy)

    ld__ixl_e
    ld__ixh_d

    ld w_vx(ix), c
    ld w_vy(ix), b

    ld a, #cooldown_changing_weapon
    ld (changing_weapon), a
    ret

__cooldown:
    dec a
    ld (changing_weapon), a
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  GAME_MANAGER::shot
;;		When the player press space he shots.
;;      The player will be able to fire again when the current bullet is destroyed. 	
;;  INPUT:
;;      C: X coordinate of the entity.
;;      D: Y coordinate of the entity.
;;      IY: Pointer to gun.
;;  MODIFY:
;;      A, DE, BC, IX, IY
;;  RETURNS:
;;
game_manager_shot::
    ;;  The position of the entity is saved to put it to the bullet.
    ld c, e_x(iy)
    ld b, e_y(iy)

    ;;  IY = Pointer to the current weapon of the entity.
    ld e, e_equipped_weapon_l(iy)
    ld d, e_equipped_weapon_h(iy)
    ld__iyl_e
    ld__iyh_d

    ;;  When the gun is recovered it will be able to fire again.
    ld a, w_recover_cadence(iy)
    or a
    call nz, game_manager_shot_again
    ret nz

    ;;  Weapon cadence recovery.
    inc a
    ld w_recover_cadence(iy), a

    ;;  The bullet array is filled according to the equipped weapon.
    call game_manager_setBulletSpeeds

    ;;  A = Number of bullets to create.
    ld a, w_burst(iy)
    ld hl, #_bullet_array

__loop:
    push hl ;;  #_bullet_array is saved to iterate over it.
    push af ;;  The bullet counter to create is saved.
    push bc ;;  The x, y position is saved where the bullets will originate.
    ld hl, #shot
    call entity_manager_create
    pop bc
    pop af
    pop hl

    ;;  The speed of the bullet array is obtained and sabed in created bullet.
    ld e, (hl)
    ld e_vx(ix), e
    inc hl
    ld e, (hl)
    ld e_vy(ix), e
    inc hl

    ;;  Bullet coordinates are set.
    ld e_x(ix), c
    ld e_y(ix), b
    
    ;;  A self-destruct counter is set for the bullet based on the maximun distance from the weapon.
    ld e, w_distance(iy)
    ld e_ai_counter(ix), e

    dec a
    jr nz, __loop
    ret
    
;;==============================================================================================================================
;;==============================================================================================================================
;;  ENTITY MANAGEMENT FUNCTIONS
;;==============================================================================================================================
;;==============================================================================================================================
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  GAME_MANAGER::entity_destroy
;;		If the entity to destroy is a shot, the counter is reset
;;      so that it can be shot again.
;;  INPUT:
;;		IX: Pointer to entity
;;  MODIFY:
;;      A
;;  RETURNS:
;; 
game_manager_entity_destroy::
    call entity_manager_set4destruction
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  GAME_MANAGER::generate_entity
;;      Spawn an entity though the enemy generator.
;;  INPUT:
;;		IX: Pointer to entity.
;;  MODIFY: A, E
;;  RETURNS:
;;
game_manager_generate_enemy::
    ld iy, #generatedEnemy

    push ix
    ld hl, #generatedEnemy
    call entity_manager_create
    ld e_ai_parent_st(ix), #e_ai_st_attack
    ld e_ai_st(ix), #e_ai_st_processing
    pop ix
    ret