;; ___  ___                 ______                _    
;; |  \/  |                 | ___ \              | |   
;; | .  . | ___   ___  _ __ | |_/ /_ __ ___  __ _| | __
;; | |\/| |/ _ \ / _ \| '_ \| ___ \ '__/ _ \/ _` | |/ /
;; | |  | | (_) | (_) | | | | |_/ / | |  __/ (_| |   < 
;; \_|  |_/\___/ \___/|_| |_\____/|_|  \___|\__,_|_|\_\
;;                                                                                                       
;; Copyright (C) 2020  
;; This file is part of the Amstrad game: MoonBreak
;;
;; Authors:
;; Ricardo Antonio, Ruiz Falcó (@R1ckyman) (ricky.rfr@gmail.com)
;; Alejandro Vicente, Rodríguez Segado (@VicenteSegado) (alejandrors13091993@gmail.com)
;; Enrique Antonio, Cores Rodríguez (@QuiqueCores) (encorod@gmail.com)
;;
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;; 
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;; 
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>
;;
;;  HUD MANAGER
;;
.module HUD_Manager
.include "hud_manager.h.s"
.include "cpctelera.h.s"

.globl _sp_hud_init_0
.globl _sp_hud_init_1
.globl _sp_energy_0
.globl _sp_energy_1
.globl _sp_o2_0
.globl _sp_o2_1
.globl _sp_o2_2
.globl _sp_o2_3
.globl _sp_o2_4
.globl _sp_pieces_0
.globl _sp_pieces_1
.globl _sp_pieces_2
.globl _sp_pieces_3
.globl _sp_pieces_4
.globl _sp_pieces_5

.globl cpct_drawSprite_asm

energy_array:
    .dw #0xDEE0+#46 ;; SCR. PTR. ENERGY CELL (0,2) 
    .dw #0xDEE0+#39 ;; SCR. PTR. ENERGY CELL (0,1)
    .dw #0xDEE0+#32 ;; SCR. PTR. ENERGY CELL (0,0)
    .dw #0xCF80+#46 ;; SCR. PTR. ENERGY CELL (1,2)
    .dw #0xCF80+#39 ;; SCR. PTR. ENERGY CELL (1,1)
    .dw #0xCF80+#32 ;; SCR. PTR. ENERGY CELL (1,0)

energy_ptr:         ;; PTR. INDEX OF ENERGY_ARRAY (needs to be initialized)
    .dw #0x0000

pieces_array:
    .dw _sp_pieces_0 ;; PTR. SPRITE 1 PIECE OBTAINED
    .dw _sp_pieces_1 ;; PTR. SPRITE 2 PIECES OBTAINED 
    .dw _sp_pieces_2 ;; PTR. SPRITE 3 PIECES OBTAINED 
    .dw _sp_pieces_3 ;; PTR. SPRITE 4 PIECES OBTAINED 
    .dw _sp_pieces_4 ;; PTR. SPRITE 5 PIECES OBTAINED 
    .dw _sp_pieces_5 ;; PTR. SPRITE ALL PIECES OBTAINED 

pieces_ptr:         ;; ;; PTR. INDEX OF PIECES_ARRAY (needs to be initialized)
    .dw #0x0000

o2_array:
    .dw _sp_o2_3 ;; PTR. SPRITE O2 AVERAGE
    .dw _sp_o2_2 ;; PTR. SPRITE O2 LOW
    .dw _sp_o2_1 ;; PTR. SPRITE O2 VERY LOW
    .dw _sp_o2_0 ;; PTR. SPRITE O2 EMPTY

o2_ptr:         ;; ;; PTR. INDEX OF O2_ARRAY (needs to be initialized)
    .dw #0x0000

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Initializes the HUD manager (draws initial state)
;;  MODIFIES:
;;      - AF, BC, DE, HL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
hud_manager_init::
    ld hl, #energy_array    ;; /
    ld (#energy_ptr), hl    ;; \ INITIALIZING ENERGY_PTR

    ld hl, #pieces_array    ;; /
    ld (#pieces_ptr), hl    ;; \ INITIALIZING PIECES_PTR

    ld hl, #o2_array    ;; /
    ld (#o2_ptr), hl    ;; \ INITIALIZING O2_PTR

    ld de, #hud_scr_ptr
	ld hl, #_sp_hud_init_0
	ld c, #hud_init_w
	ld b, #hud_init_h
	call cpct_drawSprite_asm

    ld de, #hud_scr_ptr+#hud_init_w
	ld hl, #_sp_hud_init_1
	ld c, #hud_init_w
	ld b, #hud_init_h
	call cpct_drawSprite_asm

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Increases HP (energy) in HUD (¡IMPORTANT!: Does not controll overflow)
;;  MODIFIES:
;;      - AF, BC, DE, HL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
hud_manager_inc_hp::
    ld hl, (#energy_ptr)
    dec hl
    dec hl
    ld (#energy_ptr), hl
    ld e, (hl)
    inc hl
    ld d, (hl)

	ld hl, #_sp_energy_1
	ld c, #energy_w
	ld b, #energy_h
	call cpct_drawSprite_asm

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Decreases HP (energy) in HUD (¡IMPORTANT!: Does not controll overflow)
;;  MODIFIES:
;;      - AF, BC, DE, HL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
hud_manager_dec_hp::
    ld hl, (#energy_ptr)
    ld e, (hl)
    inc hl
    ld d, (hl)

	ld hl, #_sp_energy_0
	ld c, #energy_w
	ld b, #energy_h
	call cpct_drawSprite_asm

    ld hl, (#energy_ptr)
    inc hl
    inc hl
    ld (#energy_ptr), hl

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Adds ship pieces in HUD (¡IMPORTANT!: Does not controll overflow)
;;  MODIFIES:
;;      - AF, BC, DE, HL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
hud_manager_add_piece::
    ld hl, (#pieces_ptr)
    ld e, (hl)
    inc hl
    ld d, (hl)
    ld hl, #pieces_scr_ptr
    ex de,hl
	ld c, #pieces_w
	ld b, #pieces_h
	call cpct_drawSprite_asm

    ld hl, (#pieces_ptr)
    inc hl
    inc hl
    ld (#pieces_ptr), hl

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Decreases O2 in HUD (¡IMPORTANT!: Does not controll overflow)
;;  MODIFIES:
;;      - AF, BC, DE, HL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
hud_manager_dec_o2::
    ld hl, (#o2_ptr)
    ld e, (hl)
    inc hl
    ld d, (hl)
    ld hl, #o2_scr_ptr
    ex de,hl
	ld c, #o2_w
	ld b, #o2_h
	call cpct_drawSprite_asm

    ld hl, (#o2_ptr)
    inc hl
    inc hl
    ld (#o2_ptr), hl

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Restores O2 to initial state in HUD (¡IMPORTANT!: Does not controll overflow)
;;  MODIFIES:
;;      - AF, BC, DE, HL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
hud_manager_restore_o2::
    ld de, #_sp_o2_4
    ld hl, #o2_scr_ptr
    ex de,hl
	ld c, #o2_w
	ld b, #o2_h
	call cpct_drawSprite_asm

    ld hl, #o2_array    ;; /
    ld (#o2_ptr), hl    ;; \ INITIALIZING O2_PTR

    ret