;; ___  ___                 ______                _    
;; |  \/  |                 | ___ \              | |   
;; | .  . | ___   ___  _ __ | |_/ /_ __ ___  __ _| | __
;; | |\/| |/ _ \ / _ \| '_ \| ___ \ '__/ _ \/ _` | |/ /
;; | |  | | (_) | (_) | | | | |_/ / | |  __/ (_| |   < 
;; \_|  |_/\___/ \___/|_| |_\____/|_|  \___|\__,_|_|\_\
;;                                                                                                       
;; Copyright (C) 2020  
;; This file is part of the Amstrad game: MoonBreak
;;
;; Authors:
;; Ricardo Antonio, Ruiz Falcó (@R1ckyman) (ricky.rfr@gmail.com)
;; Alejandro Vicente, Rodríguez Segado (@VicenteSegado) (alejandrors13091993@gmail.com)
;; Enrique Antonio, Cores Rodríguez (@QuiqueCores) (encorod@gmail.com)
;;
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;; 
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;; 
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>
.include "cpctelera.h.s"
.include "assets/screens/s_menu_z.h.s"
.include "assets/screens/s_controls_z.h.s"
.include "assets/screens/s_credits_z.h.s"
.globl cpct_isKeyPressed_asm
.globl cpct_isAnyKeyPressed_asm
.globl cpct_scanKeyboard_asm
.globl cpct_zx7b_decrunch_s_asm

.module Menu_Manager

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Initializes the menu loop
;;  MODIFIES:
;;      - HL, DE, AF, AF'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
menu_manager_init::
    ld hl, #_s_menu_z_end
    ld de, #0xFFFF
    call cpct_zx7b_decrunch_s_asm
    ld a, #5
    call wait_A_dseconds

menu_manager_input:                 ;; Main menu input manager
    call cpct_scanKeyboard_asm
    ld hl, #Key_1
    call cpct_isKeyPressed_asm
    jr z, K1_NotPressed

K1_Pressed:
    ret

K1_NotPressed:
    ld hl, #Key_2
    call cpct_isKeyPressed_asm
    jr z, K2_NotPressed

K2_Pressed:
    ld hl, #_s_controls_z_end
    ld de, #0xFFFF
    call cpct_zx7b_decrunch_s_asm
    ld a, #5
    call wait_A_dseconds
    jr menu_manager_anyinput

K2_NotPressed:
    ld hl, #Key_3
    call cpct_isKeyPressed_asm
    jr z, K3_NotPressed

K3_Pressed:
    ld hl, #_s_credits_z_end
    ld de, #0xFFFF
    call cpct_zx7b_decrunch_s_asm
    ld a, #5
    call wait_A_dseconds
    jr menu_manager_anyinput

K3_NotPressed:
    jr menu_manager_input

menu_manager_anyinput:          ;; Menu screens input manager
    call cpct_scanKeyboard_asm
    call cpct_isAnyKeyPressed_asm
    jr z, Any_NotPressed

Any_Pressed:
    jr menu_manager_init

Any_NotPressed:
    jr menu_manager_anyinput

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Waits A deciseconds (A * 0.1s)
;;  MODIFIES:
;;      - AF, AF'
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
wait_A_dseconds:
    ex af, af'
    ld a, #0x1E
inner_loop:
    halt
    dec a
    jr nz, inner_loop
    ex af, af'
    dec a
    jr nz, wait_A_dseconds
    ret