;; ___  ___                 ______                _    
;; |  \/  |                 | ___ \              | |   
;; | .  . | ___   ___  _ __ | |_/ /_ __ ___  __ _| | __
;; | |\/| |/ _ \ / _ \| '_ \| ___ \ '__/ _ \/ _` | |/ /
;; | |  | | (_) | (_) | | | | |_/ / | |  __/ (_| |   < 
;; \_|  |_/\___/ \___/|_| |_\____/|_|  \___|\__,_|_|\_\
;;                                                                                                       
;; Copyright (C) 2020  
;; This file is part of the Amstrad game: MoonBreak
;;
;; Authors:
;; Ricardo Antonio, Ruiz Falcó (@R1ckyman) (ricky.rfr@gmail.com)
;; Alejandro Vicente, Rodríguez Segado (@VicenteSegado) (alejandrors13091993@gmail.com)
;; Enrique Antonio, Cores Rodríguez (@QuiqueCores) (encorod@gmail.com)
;;
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;; 
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;; 
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
;;  PATROL_MANAGER
;;      Administers the patrols assigning tours to the entities that require it.
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================

.include "components_manager.h.s"
.include "cmp/entity.h.s"
.include "patrol_manager.h.s"
.include "cpctelera.h.s"

.module Patrol_Manager

_patrol_01:
    .db 30, 90
    .db 20, 90
    .db patrol_invalid_move_x
    .dw _patrol_01

_patrol_02:
    .db 50, 100
    .db 70, 100
    .db patrol_invalid_move_x
    .dw _patrol_02

_patrol_03:
    .db 60, 20
    .db 60, 40
    .db patrol_invalid_move_x
    .dw _patrol_03

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  PATROL_MANAGER::Init
;;      Assigns a patrol to the entities that require it.
;;  INPUT:
;;  MODIFY:
;;       B, DE, HL, A, IX
;;
patrol_manager_init::
    ld a, #e_cmpID_AI
    call components_manager_getArrayHL

    ld b, #0

    _loop:
        ;;  Select the entity pointer.
        ld e, (hl)
        inc hl
        ld d, (hl)
        inc hl

        ;;  Ends loop when pointer is to null.
        ld a, e
        or d
        ret z

        push hl

        ld__ixl_e
        ld__ixh_d

        ;;  Check if entity has patrol
        ld a, e_ai_st(ix)
        cp #e_ai_st_patrol_selector
        jr nz, _no_patrol_entity

        ;;  Entity with patrol
        ld a, b
        call patrol_manager_get
        ld  e_ai_patrol_step_l(ix), l
        ld  e_ai_patrol_step_h(ix), h
        inc b

_no_patrol_entity:
        pop hl
        jr _loop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  PATROL_MANAGER::get
;;      Returns a poiner to the start of the patrol passed by parameter.
;;  INPUT:
;;  MODIFY:
;;       HL
;;
patrol_manager_get::
    cp #0
    jr nz, _p2

_p1:
    ld hl, #_patrol_01
    ret

_p2:
    cp #1
    jr nz, _p3
    ld hl, #_patrol_02
    ret

_p3:
    ld hl, #_patrol_03
    ret