;; ___  ___                 ______                _    
;; |  \/  |                 | ___ \              | |   
;; | .  . | ___   ___  _ __ | |_/ /_ __ ___  __ _| | __
;; | |\/| |/ _ \ / _ \| '_ \| ___ \ '__/ _ \/ _` | |/ /
;; | |  | | (_) | (_) | | | | |_/ / | |  __/ (_| |   < 
;; \_|  |_/\___/ \___/|_| |_\____/|_|  \___|\__,_|_|\_\
;;                                                                                                       
;; Copyright (C) 2020  
;; This file is part of the Amstrad game: MoonBreak
;;
;; Authors:
;; Ricardo Antonio, Ruiz Falcó (@R1ckyman) (ricky.rfr@gmail.com)
;; Alejandro Vicente, Rodríguez Segado (@VicenteSegado) (alejandrors13091993@gmail.com)
;; Enrique Antonio, Cores Rodríguez (@QuiqueCores) (encorod@gmail.com)
;;
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;; 
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;; 
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>
.include "man/components_manager.h.s"
.include "man/entity_manager.h.s"
.include "cmp/entity.h.s"
.include "cpctelera.h.s"

;;  SYS/AI_CONTROL_SYSTEM/
.include "ai_control_system/selectors_and_parent_nodes.h.s"
.include "ai_control_system/utilities.h.s"
.include "ai_control_system/behavior_attack.h.s"
.include "ai_control_system/behavior_turret.h.s"
.include "ai_control_system/behavior_move_to.h.s"
.include "ai_control_system/simple_tasks.h.s"

.module AI_Control_System

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  AI_CONTROL_SYSTEM::Init
;;      Initialize the ai system with an array of pointers 
;;      containing the entities with ai.
;;  INPUT:
;;  MODIFY:
;;       HL
;;
ai_control_system_init::
    ld a, #e_cmpID_AI
    call components_manager_getArrayHL
    ld  (_ent_array_ptr), hl
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  AI_CONTTROL_SYSTEM::Update
;;      Updates all AI entities
;;  INPUT:
;;      HL: Pointer to array of entity pointers with AI
;;  MODIFY:
;;      HL, DE, A, IX
;;
ai_control_system_update::

_ent_array_ptr = . + 1
    ld  hl, #0x0000

_loop:
    ;;  Select the pointer to the entity with AI and prepare the next position for the next iteration.
    ld e, (hl)
    inc hl
    ld d, (hl)
    inc hl

    ;;  The entities are finished traversing when find a pointer to null.
    ld a, e
    or d
    ret z

    push hl

    ld__ixl_e
    ld__ixh_d

    ld a, e_ai_st(ix)
    xor #e_ai_st_bullet
    call z, ai_control_system_behavior_autodestroy

    ld a, e_ai_st(ix)
    xor #e_ai_st_bullet
    call z, ai_control_system_behavior_edge_destruction

    ld a, e_ai_st(ix)
    xor #e_ai_st_turret
    call z, ai_control_system_behavior_turret

    ld a, e_ai_st(ix)
    xor #e_ai_st_guard
    call z, ai_control_system_behavior_guard

    ld a, e_ai_st(ix)
    xor #e_ai_st_guard_selector
    call z, ai_control_system_behavior_guard_selector

    ld a, e_ai_st(ix)
    xor #e_ai_st_patrol
    call z, ai_control_system_behavior_patrol

    ld a, e_ai_st(ix)
    xor #e_ai_st_move_to
    call z, ai_control_system_behavior_move_to

    ld a, e_ai_st(ix)
    xor #e_ai_st_patrol_selector
    call z, ai_control_system_behavior_patrol_selector
    
    ld a, e_ai_st(ix)
    xor #e_ai_st_attack
    call z, ai_control_system_behavior_attack

    ld a, e_ai_st(ix)
    xor #e_ai_st_processing
    call z, ai_control_system_behavior_processing

    ld a, e_ai_st(ix)
    xor #e_ai_st_escape
    call z, ai_control_system_behavior_escape

    ld a, e_ai_st(ix)
    xor #e_ai_st_heal
    call z, ai_control_system_behavior_heal

    pop hl

    jr _loop
