;; ___  ___                 ______                _    
;; |  \/  |                 | ___ \              | |   
;; | .  . | ___   ___  _ __ | |_/ /_ __ ___  __ _| | __
;; | |\/| |/ _ \ / _ \| '_ \| ___ \ '__/ _ \/ _` | |/ /
;; | |  | | (_) | (_) | | | | |_/ / | |  __/ (_| |   < 
;; \_|  |_/\___/ \___/|_| |_\____/|_|  \___|\__,_|_|\_\
;;                                                                                                       
;; Copyright (C) 2020  
;; This file is part of the Amstrad game: MoonBreak
;;
;; Authors:
;; Ricardo Antonio, Ruiz Falcó (@R1ckyman) (ricky.rfr@gmail.com)
;; Alejandro Vicente, Rodríguez Segado (@VicenteSegado) (alejandrors13091993@gmail.com)
;; Enrique Antonio, Cores Rodríguez (@QuiqueCores) (encorod@gmail.com)
;;
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;; 
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;; 
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
;;  BEHAVIOR_MOVE_TO
;;      Change the speeds of the entity to direct it to a coordinate.
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
.module Ai_Behavior_Move_To

;;  CMP/
.include "cmp/entity.h.s"
;;  SYS/AI_CONTROL_SYSTEM/
.include "sys/ai_control_system/utilities.h.s"
.include "sys/ai_control_system/behavior_attack.h.s"

;;==============================================================================================================================
;;==============================================================================================================================
;;  AI_CONTROL_SYSTEM::behavior_move_to
;;      Change the speeds of the entity to direct it to a coordinate.
;;  INPUT:
;;      IX: Pointer to entity with AI.
;;  MODIFY:
;;      A
;;==============================================================================================================================
;;==============================================================================================================================
ai_control_system_behavior_move_to::
    ;;  The short range class stops and fires if the player is within its firing range.
    ld a, e_class(ix)
    and #e_class_short_range
    call nz, ai_control_system_short_range_attack
    or a
    jr nz, __interrup_move_to

    ld a, e_ai_aim_x(ix)
    sub e_x(ix)
    jr nc, #__objx_greater_or_equal

__objx_lesser:
    ld e_vx(ix), #-1
    jr __endif_x

__objx_greater_or_equal:
    jr z, #__arrived_x
    ld e_vx(ix), #1
    jr __endif_x

__arrived_x:
    ld e_vx(ix), #0
    
__endif_x:
    ld a, e_ai_aim_y(ix)
    sub e_y(ix)
    jr nc, #__objy_greater_or_equal

__objy_lesser:
    ld e_vy(ix), #-1
    jr __endif_y

__objy_greater_or_equal:
    jr z, #__arrived_y
    ld e_vy(ix), #1
    jr __endif_y

__arrived_y:
    ld e_vy(ix), #0
    ld a, e_vx(ix)
    or a
    jr nz, #__endif_y

__interrup_move_to:
        ;;  X and Y are both 0.
        ld e_ai_st(ix), #e_ai_st_processing
        ld e_ai_pre_st(ix), #e_ai_st_move_to

__endif_y:
    ret