;; ___  ___                 ______                _    
;; |  \/  |                 | ___ \              | |   
;; | .  . | ___   ___  _ __ | |_/ /_ __ ___  __ _| | __
;; | |\/| |/ _ \ / _ \| '_ \| ___ \ '__/ _ \/ _` | |/ /
;; | |  | | (_) | (_) | | | | |_/ / | |  __/ (_| |   < 
;; \_|  |_/\___/ \___/|_| |_\____/|_|  \___|\__,_|_|\_\
;;                                                                                                       
;; Copyright (C) 2020  
;; This file is part of the Amstrad game: MoonBreak
;;
;; Authors:
;; Ricardo Antonio, Ruiz Falcó (@R1ckyman) (ricky.rfr@gmail.com)
;; Alejandro Vicente, Rodríguez Segado (@VicenteSegado) (alejandrors13091993@gmail.com)
;; Enrique Antonio, Cores Rodríguez (@QuiqueCores) (encorod@gmail.com)
;;
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;; 
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;; 
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
;;  BEHAVIOR_TURRET
;;      When the entity has this behavior it shoots horizontally and vertically at the enemy at any distance.
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
.module Ai_Behavior_Turret

;;  MAN/
.include "man/entity_manager.h.s"
.include "man/game_manager.h.s"
;;  CMP/
.include "cmp/entity.h.s"
.include "cmp/weapon.h.s"
;;  SYS/AI_CONTROL_SYSTEM/
.include "sys/ai_control_system/utilities.h.s"
;;  CPCTELERA/
.include "cpctelera.h.s"

;;==============================================================================================================================
;;==============================================================================================================================
;;  AI_CONTROL_SYSTEM::BEHAVIOR_TURRET
;;      The entity with AI shoots the player if it detects horizontally or vertically.
;;      Does not fire in diagonal.
;;  INPUT:
;,      IX: Pointer to AI entity.
;;  MODIFY:
;;      IY, DE, B, A
;;  RETURNS:
;;==============================================================================================================================
;;==============================================================================================================================
ai_control_system_behavior_turret::
    ;;  If the player is in shooting range, they will get to 1.
    ld b, #0x00

    ;;  To save the speed of the shoot in case the player is seen.
    ld de, #nullptr

    ;;  IY = Pointer to player.
    call entity_manager_getPlayerPositionIY

    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;  PLAYER TO SHOT ON X AXIS
    ;;      Checks if the entity with ia has a shot at the player on the x axis.
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;  Player at x-axis shot entering from the left.
    ld a, e_endx(iy)
    sub e_x(ix)
    jr c, __player_not_shot_x_axis

    ;;  Player at x-axis shot entering from the right.
    ld a, e_endx(ix)
    sub e_x(iy)
    jr c, __player_not_shot_x_axis

;;  If the player is in sight on the x-axis the entity with ai shoots.
__player_to_shot_x_axis:
    inc b
    ld a, e_y(ix)
    sub e_y(iy)
    jr c, __player_down

    __player_up:
    ld d, #-4
    jr __player_not_shot_x_axis

    __player_down:
    ld d, #4

__player_not_shot_x_axis:
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;;  PLAYER TO SHOT ON Y AXIS
    ;;      Checks if the entity with ia has a shot at the player on the y axis.
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

    ;;  Player at x-axis shot entering from the above.
    ld a, e_endy(iy)
    sub e_y(ix)
    jr c, __player_not_shot_y_axis
    
    ;;  Player at y-axis shot entering from below.
    ld a, e_endy(ix)
    sub e_y(iy)
    jr c, __player_not_shot_y_axis

;;  If the player is in sight on the y-axis the entity with ai shoots.
__player_to_shot_y_axis:
    inc b
    ld a, e_x(ix)
    sub e_x(iy)
    jr c, __player_right

    __player_left:
    ld e, #-2
    jr __player_not_shot_y_axis

    __player_right:
    ld e, #2

__player_not_shot_y_axis:
    ;;  If B is zero the player has not been seen and will not fire.
    ld a, b
    or a
    ret z

    ;;  If the player has been seen the wepon speed (bullet speed) and the gun is fired.
    ld c, e_equipped_weapon_l(ix)
    ld b, e_equipped_weapon_h(ix)

    ld__iyl_c
    ld__iyh_b

    ld w_vx(iy), e
    ld w_vy(iy), d

    ld__e_ixl
    ld__d_ixh

    ld__iyl_e
    ld__iyh_d

    ld e_ai_st(ix), #e_ai_st_processing
    call game_manager_shot

    ret