;; ___  ___                 ______                _    
;; |  \/  |                 | ___ \              | |   
;; | .  . | ___   ___  _ __ | |_/ /_ __ ___  __ _| | __
;; | |\/| |/ _ \ / _ \| '_ \| ___ \ '__/ _ \/ _` | |/ /
;; | |  | | (_) | (_) | | | | |_/ / | |  __/ (_| |   < 
;; \_|  |_/\___/ \___/|_| |_\____/|_|  \___|\__,_|_|\_\
;;                                                                                                       
;; Copyright (C) 2020  
;; This file is part of the Amstrad game: MoonBreak
;;
;; Authors:
;; Ricardo Antonio, Ruiz Falcó (@R1ckyman) (ricky.rfr@gmail.com)
;; Alejandro Vicente, Rodríguez Segado (@VicenteSegado) (alejandrors13091993@gmail.com)
;; Enrique Antonio, Cores Rodríguez (@QuiqueCores) (encorod@gmail.com)
;;
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;; 
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;; 
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
;;  BEHAVIOR_ATTACK
;;      Determines the actions of an entity when in enters attack mode.
;;      These actions will depend on the entity class and the state it is in.
;;      
;;      -BHEAVIOR_AUTODESTROY:            If the shot hits the edge it self-destructs.
;;      -BEHAVIOR_EDGE_DESTRUCTION:       The entity heals until reaching its maximum life. 
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
;;==============================================================================================================================
.module Ai_Simple_Tasks

;;  MAN/
.include "man/entity_manager.h.s"
.include "man/game_manager.h.s"
;;  CMP/
.include "cmp/entity.h.s"
;;  SYS/
.include "sys/physics_system.h.s"

;;==============================================================================================================================
;;==============================================================================================================================
;;  AI_CONTROL_SYSTEM::behavior_autodestroy
;;      The entity is destroyed at the end of its counter.
;;  INPUT:
;;      IX: Pointer to entity with AI.
;;  MODIFY:
;;  RETURNS:
;;==============================================================================================================================
;;==============================================================================================================================
ai_control_system_behavior_autodestroy::
    ld a, e_ai_counter(ix)
    dec a
    ld e_ai_counter(ix), a
    call z, game_manager_entity_destroy
    ret

;;==============================================================================================================================
;;==============================================================================================================================
;;  AI_CONTROL_SYSTEM::behavior_edge_destruction
;;      If the shot hits the edge it self-destructs.
;;  INPUT:
;;      IX: Pointer to entity with AI.
;;  MODIFY:
;;  RETURNS:
;;==============================================================================================================================
;;==============================================================================================================================
ai_control_system_behavior_edge_destruction::
    ;;  If the bullet goes off the horizontal axis it is destroyed.
    call physics_system_check_xaxis
    jr  nc, _invalid_coord
    ;;  If the bullet goes off the vertical axis it is destroyed.
    call physics_system_check_yaxis
    ret c

_invalid_coord:
    call game_manager_entity_destroy
    ret