;; ___  ___                 ______                _    
;; |  \/  |                 | ___ \              | |   
;; | .  . | ___   ___  _ __ | |_/ /_ __ ___  __ _| | __
;; | |\/| |/ _ \ / _ \| '_ \| ___ \ '__/ _ \/ _` | |/ /
;; | |  | | (_) | (_) | | | | |_/ / | |  __/ (_| |   < 
;; \_|  |_/\___/ \___/|_| |_\____/|_|  \___|\__,_|_|\_\
;;                                                                                                       
;; Copyright (C) 2020  
;; This file is part of the Amstrad game: MoonBreak
;;
;; Authors:
;; Ricardo Antonio, Ruiz Falcó (@R1ckyman) (ricky.rfr@gmail.com)
;; Alejandro Vicente, Rodríguez Segado (@VicenteSegado) (alejandrors13091993@gmail.com)
;; Enrique Antonio, Cores Rodríguez (@QuiqueCores) (encorod@gmail.com)
;;
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;; 
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;; 
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>
.include "man/entity_manager.h.s"
.include "man/game_manager.h.s"
.include "cpctelera.h.s"
.include "cmp/entity.h.s"
.include "cmp/weapon.h.s"

.globl cpct_isKeyPressed_asm
.globl cpct_scanKeyboard_asm

.module Input_System

input_system_init::
  ret

input_system_update::
    call entity_manager_getPlayerPositionIY

    ;; Reset player velocities
    ld e_vx(iy), #0
    ld e_vy(iy), #0
    
    ;; Scan the keyborad in case a key has been pressed
    call cpct_scanKeyboard_asm

_check_left:
    ld hl, #Key_O
    call cpct_isKeyPressed_asm
    jr nz, _left_pressed
    ld hl, #Joy0_Left
    call cpct_isKeyPressed_asm
    jr nz, _left_pressed
    jr _check_right
_left_pressed:
    ld e_vx(iy), #-1
_check_right:
    ld hl, #Key_P
    call cpct_isKeyPressed_asm
    jr nz, _right_pressed
    ld hl, #Joy0_Right
    call cpct_isKeyPressed_asm
    jr nz, _right_pressed
    jr _check_up
_right_pressed:
    ld e_vx(iy), #1
_check_up:
    ld hl, #Key_Q
    call cpct_isKeyPressed_asm
    jr nz, _up_pressed
    ld hl, #Joy0_Up
    call cpct_isKeyPressed_asm
    jr nz, _up_pressed
    jr _check_down
_up_pressed:
    ld e_vy(iy), #-2
_check_down:
    ld hl, #Key_A
    call cpct_isKeyPressed_asm
    jr nz, _down_pressed
    ld hl, #Joy0_Down
    call cpct_isKeyPressed_asm
    jr nz, _down_pressed
    jr _finished_movement_check
_down_pressed:
    ld e_vy(iy), #2
_finished_movement_check:
    ;;  It is checked if the player's speeds are different from zero
    ;;  in case he has changed position.
    ld a, e_vx(iy)
    or a
    jr nz, __upgrade_weapon_speed

    ld a, e_vy(iy)
    or a
    jr z, __not_upgrade_weapon_speed

    ;;  If the player's sepeeds have been altered, the speed of the weapon
    ;;  that is equipped is changed.
    ;;  Weapon speed indicates the direction the buller will takw when fired.
__upgrade_weapon_speed:
    ;;  ED = Pointer to the player's current weapon.
    ld e, e_equipped_weapon_l(iy)
    ld d, e_equipped_weapon_h(iy)

    ;; IX = ED
    ld__ixl_e
    ld__ixh_d

    ;;  Update the weapon speeds with the player's speed x 2.
    ld a, e_vx(iy)
    add a
    ld w_vx(ix), a

    ld a, e_vy(iy)
    add a
    ld w_vy(ix), a

__not_upgrade_weapon_speed:
    ld hl, #Key_Space
    call cpct_isKeyPressed_asm
    jr nz, _fire_pressed
    ld hl, #Joy0_Fire1
    call cpct_isKeyPressed_asm
    jr nz, _fire_pressed
    jr _check_swap_weapon
_fire_pressed:
    call game_manager_shot
_check_swap_weapon:
    ld hl, #Key_Enter
    call cpct_isKeyPressed_asm
    jr nz, _swap_weapon_pressed
    ld hl, #Joy0_Fire2
    call cpct_isKeyPressed_asm
    jr nz, _swap_weapon_pressed
    jr _check_pause_menu
_swap_weapon_pressed:
    call game_manager_change_weapon
_check_pause_menu:
    ld hl, #Key_Esc
    call cpct_isKeyPressed_asm
    jr nz, _pause_menu_pressed
    jr _finish_checks
_pause_menu_pressed:
    ;; Set the state to pause menu
    ld a, #3
    call game_manager_set_state
    
_finish_checks:
    ret