;; ___  ___                 ______                _    
;; |  \/  |                 | ___ \              | |   
;; | .  . | ___   ___  _ __ | |_/ /_ __ ___  __ _| | __
;; | |\/| |/ _ \ / _ \| '_ \| ___ \ '__/ _ \/ _` | |/ /
;; | |  | | (_) | (_) | | | | |_/ / | |  __/ (_| |   < 
;; \_|  |_/\___/ \___/|_| |_\____/|_|  \___|\__,_|_|\_\
;;                                                                                                       
;; Copyright (C) 2020  
;; This file is part of the Amstrad game: MoonBreak
;;
;; Authors:
;; Ricardo Antonio, Ruiz Falcó (@R1ckyman) (ricky.rfr@gmail.com)
;; Alejandro Vicente, Rodríguez Segado (@VicenteSegado) (alejandrors13091993@gmail.com)
;; Enrique Antonio, Cores Rodríguez (@QuiqueCores) (encorod@gmail.com)
;;
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.
;; 
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;; 
;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>
.module Render_System

.include "man/entity_manager.h.s"
.include "man/map_manager.h.s"
.include "man/animation_manager.h.s"
.include "cpctelera.h.s"
.include "cmp/entity.h.s"

.globl cpct_getScreenPtr_asm
.globl cpct_drawSprite_asm
.globl cpct_drawSolidBox_asm
.globl cpct_setVideoMode_asm
.globl cpct_setPalette_asm

.globl _g_palette

screen_start = 0xC000

;;------------------------------------------------------------------------------------------------------------------------------
;;------------------------------------------------------------------------------------------------------------------------------


;;  INPUT:
;;    IX: Pointer to entity to update
;;  MODIFIES:
;;    A, DE, C, B, HL
;;  Paint the sprite on screen
render_system_update::
	ld (_end_counter), a

_loop:
	call animation_manager_update
	push ix
	ld a, e_type(ix)
	cp #e_type_dead
	jr nz, #_alive_entity
	call map_manager_restore
	call map_manager_set_to_restore
	pop ix
	jr dont_draw_sprite
	_alive_entity:

	call map_manager_restore
	pop ix

draw_sprite:

	ld de, #screen_start
	ld c, e_x(ix)
	ld b, e_y(ix)
	call cpct_getScreenPtr_asm

	ex de, hl
	ld l, e_spr_l(ix)
	ld h, e_spr_h(ix)
	ld c, e_w(ix)
	ld b, e_h(ix)
	call cpct_drawSprite_asm

dont_draw_sprite:
_end_counter = . + 1
	ld a, #0
	dec a
	ret z

	ld (_end_counter), a
    ld e, e_ptr_l(ix)
    ld d, e_ptr_h(ix)
    ld__ixl_e
    ld__ixh_d

	jr _loop


;;  INPUT:
;;    HL: Pointer to macro
;;  MODIFIES
;;    C, HL, DE
;;  Initialize the render by setting mode 0 and the color palette
render_system_init::
	ld c, #0
	call cpct_setVideoMode_asm
	cpctm_setBorder_asm HW_BLACK
	ld hl, #_g_palette
	ld de, #16
	call cpct_setPalette_asm
	ret